package org.sfb441.tradisc.annotation;

import java.io.File;
import java.io.IOException;
import java.lang.Math;
import java.util.Vector;

import org.sfb441.tradisc.Location;
import org.sfb441.tradisc.Markable;
import org.sfb441.tradisc.Tag;
import org.sfb441.tradisc.XMLUtilities;
import org.sfb441.tradisc.inputformat.CorpusFormat;
import org.sfb441.tradisc.schema.SchemaModel;
import org.sfb441.tradisc.schema.SchemaException;
import org.w3c.dom.Document;

public class AnnotationModel {

	protected Annotator caller;

	// XML
	private Document corpus; // annotation: corpus

	// Annotation Document
	protected AnnotationDocument annDoc;

	protected AnnotationView annView;

	public AnnotationModel(Annotator annotator) {
		this.caller = annotator;
	}

	/**
	 * Baue eine neue Annotation auf.
	 * 
	 * @param corpusFile
	 *            dieses Corpus wird verwendet.
	 * @param schema
	 *            dieses Schema wird verwendet.
	 * @param tokenInfo
	 *            damit wissen wir, wo die tokens zu finden sind.
	 * @throws AnnotationException
	 */
	public void build(File corpusFile, SchemaModel schema)
			throws AnnotationException {

		corpus = XMLUtilities.parseXMLFile(corpusFile, false);
		if (corpus == null) {
			throw new AnnotationException("Couldn't parse corpus xml file "
					+ corpusFile.getName());
		}
		// init annotation document
		CorpusFormat tokenFormat = caller.getInputFormat();
		annDoc = new AnnotationDocument();
		annDoc.build(corpus, schema, tokenFormat, null);

		// create view
		annView = new AnnotationView(this);
		annView.doLayout(caller.getInputFormat());
	}

	/**
	 * Ldt eine schon bestehende Annotation in den Editor. Die Schemadaten
	 * werden in den SchemaEditor geladen
	 * 
	 * @param annotationFile
	 *            das Korpus
	 * @param annotationDataFile
	 *            die Annotationsdaten
	 * @param tokenInfo
	 *            was ist ein Token?
	 * @return das erste Element des Korpus
	 * @throws AnnotationException
	 * @throws SchemaException
	 */
	public void rebuild(File annotationFile, File annotationDataFile)
			throws AnnotationException, SchemaException {
		try {
			// Annotationsdokument laden
			CorpusFormat tokenFormat = caller.getInputFormat();
			AnnotationDocumentReader adr = new AnnotationDocumentReader(
					annotationFile, annotationDataFile, tokenFormat);
			annDoc = adr.read();

			// View wieder aufbauen. Vorhandene Tags werden bercksichtigt.
			annView = new AnnotationView(this);
			annView.doLayout(annDoc.getTokenFormat());

			// ber Annotator und GUI das neue Schema in den SchemaEditor laden.
			caller.schemaReload(annDoc.getSchema());
			caller.corpusFormatReload(annDoc.getTokenFormat());
			
		} catch (Exception e) {
			this.annDoc = null;
			throw new AnnotationException(e.getMessage());
		}
	}

	public void mergeWith(File mergeCorpusFile, File mergeDataFile)
			throws AnnotationException, SchemaException {
		try {

			AnnotationDocumentReader adr = new AnnotationDocumentReader(
					mergeCorpusFile, mergeDataFile, caller.getInputFormat());
			AnnotationDocument mergeDoc = new AnnotationDocument();
			mergeDoc = adr.read();
			annDoc.mergeWith(mergeDoc);
			annView = new AnnotationView(this);
			annView.doLayout(caller.getInputFormat());

		} catch (Exception e) {
			// this.annDoc = null;
			throw new AnnotationException(e.getMessage());
		}
	}

	public void splitAnnotation(File corpusFile1, File dataFile1,
			File corpusFile2, File dataFile2) throws Exception {
		try {
			int splint_e = annView.getCurrentElementNum();
			int splint_m = -1;
			for (int i = splint_e; i < annDoc.allElements.size(); i++) {
				Location l = annDoc.allElements.elementAt(i);
				if (l.marked){
					splint_m = annDoc.markables.indexOf(l);
					break;
				}
			}
			if(splint_m == -1){
				splint_m = annDoc.getMarkableCount();
			}
			//int splint_m = annView.getCurrentMarkableNum();
			System.err.println("splint_m: " + splint_m);
			
			System.err.println("splint_e: " + splint_e);
			// wenn man an der Position eines Markables
			if (this.getCurrentMarkableLocation().equals(
					this.getCurrentElementLocation()))
				splint_m--;
			AnnotationDocumentWriter adw = new AnnotationDocumentWriter(
					corpusFile1, dataFile1);
			adw.splitWrite(annDoc, splint_m, splint_e, true);
			adw.close();
			adw = new AnnotationDocumentWriter(corpusFile2, dataFile2);
			adw.splitWrite(annDoc, splint_m, splint_e, false);
			adw.close();
		} catch (Exception e) {
			throw new AnnotationException(e.getMessage());
		}
	}

	public void rebuildAnnotation() {
		annDoc.rebuild();
		try {
			annView.redoLayout();
		} catch (AnnotationException e) {
			System.err.println(e.getMessage());
		}
	}

	public void save(File annotationFile, File annotationDataFile)
			throws AnnotationException, SchemaException, IOException {
		AnnotationDocumentWriter adw = new AnnotationDocumentWriter(
				annotationFile, annotationDataFile);
		adw.write(annDoc);
		adw.close();
	}

	// Bewegungen im Korpus

	public Location move(String command) {
		return annView.move(command);
	}

	public Location getCurrentMarkableLocation() {
		return annView.getCurrentMarkable();
	}

	public Location getCurrentTokenLocation() {
		return annView.getCurrentToken();
	}

	public Location getCurrentElementLocation() {
		return annView.getCurrentElement();
	}

	public int getCurrentTokenNumber() {
		return annView.getCurrentTokenNum();
	}

	public boolean isCurrentPositionStart() {
		return annView.getCurrentMarkableNum() == 0;
	}

	public boolean isCurrentPositionEnd() {
		return annView.getCurrentMarkableNum() == annDoc.getMarkableCount() - 1;
	}

	// Annotation im Korpus

	public void setTagOnCurrentMarkable(Tag t) {
		System.err.println("setting tag to " + t.toString());
		System.err.println(annView.getCurrentMarkableNum());
		annDoc.setTagOnMarkableAt(t, annView.getCurrentMarkableNum());
	}

	public void setNoTagOnCurrentMarkable() {
		System.err.println("Setting no Tag.");
		System.err.println(annView.getCurrentMarkableNum());
		annDoc.setNoTagOnMarkableAt(annView.getCurrentMarkableNum());
	}

	public boolean isCurrentMarkableTagged() {
		Location l = annView.getCurrentMarkable();
		return (l.tag != null);
	}

	public Document getCorpusDocument() {
		return this.corpus;
	}

	public void setCorpusDocument(Document doc) {
		this.corpus = doc;
	}

	public int getTagCount(Tag t) {
		return annDoc.getTagCount(t);
	}

	public int getWindowTagCount(Tag t, int start, int end) {
		return annDoc.getWindowTagCount(t, start, end);
	}

	public double getDistributionFor(Vector<Markable> v, Tag t) {
		return annDoc.getDistributionFor(v, t);
	}

	public int countAnnotatedMarkables() {
		return annDoc.countAnnotatedMarkables();
	}
	
	public double getNormalizedDistributionFor(Vector<Markable> v, Tag t, int normfact) {
		if (normfact < 1) {
			return 0.0;
		} else {
			double d = ((double) Math.round((annDoc
					.getDistributionFor(v, t) / ((double) annDoc
					.getTokenCount() / (double) normfact)) * 100) / 100);
			
			return d;
		}
	}

	/*
	 * Chris: gendert, damit man auch mit Faktoren grer als die Wortzahl des
	 * Korpus normalisieren kann (sozusagen auf grere Texte hochrechnen)
	 */
	public double getNormalizedTagCount(Tag t, int constant) {
		if (constant < 1) {
			return 0.0;
		} else {
			double d = ((double) Math
					.round((annDoc.getTagCount(t) / ((double) annDoc
							.getTokenCount() / (double) constant)) * 100) / 100);
			return d;
		}
	}


	public int getWindowMarkableCount(int start, int end) {
		return annDoc.countAnnotatedMarkablesInWindow(start, end);
	}

}